package spoon.examples.fragments.processing;

import java.io.Serializable;

import spoon.processing.AbstractProcessor;
import spoon.reflect.cu.CompilationUnit;
import spoon.reflect.cu.SourceCodeFragment;
import spoon.reflect.declaration.CtClass;
import spoon.reflect.reference.CtTypeReference;
import spoon.reflect.visitor.DefaultJavaPrettyPrinter;

/**
 * When a class implements a Serializable interface, remove it and annotate the
 * class with a Serializable annotation.
 */

public class SerializableProcessor extends AbstractProcessor<CtClass<?>> {
	public void process(CtClass<?> c) {
		CompilationUnit cu=c.getPosition().getCompilationUnit();
		if(cu==null) return;
		CtTypeReference<Serializable> serializable = getFactory().Type()
				.createReference(Serializable.class);
		if (c.getSuperInterfaces().contains(serializable)) {
			c.getSuperInterfaces().remove(serializable);
			SourceCodeFragment fragment = new SourceCodeFragment();
			int classNameStart = c.getPosition().getSourceStart();
			int classDeclStart = cu.beginOfLineIndex(classNameStart);
			int classDeclEnd = cu.getOriginalSourceCode().indexOf('{',
					classNameStart);
			fragment.position = classDeclStart;
			fragment.replacementLength = classDeclEnd - classDeclStart;
			DefaultJavaPrettyPrinter pp = new DefaultJavaPrettyPrinter(
					getEnvironment());
			pp.writeModifiers(c).write("class " + c.getSimpleName());
			pp.writeGenericsParameter(c.getFormalTypeParameters());
			pp.writeExtendsClause(c).writeImplementsClause(c).write(" ");
			fragment.code = "@Serializable " + pp.getResult().toString();
			cu.addSourceCodeFragment(fragment);
		}
	}
}
